﻿using System.Collections.ObjectModel;
using System.ComponentModel;

namespace CheckBoxesInCombo
{
	public class ResourceItemCollection : ObservableCollection<ResourceItem>
	{
		public ResourceItemCollection()
		{
			this.CheckedItems = new ObservableCollection<ResourceItem>();
		}

		public ObservableCollection<ResourceItem> CheckedItems { get; private set; }

		protected override void InsertItem(int index, ResourceItem item)
		{
			this.InitializeNotifications(item);
			base.InsertItem(index, item);
		}

		protected override void RemoveItem(int index)
		{
			this.ClearNotifications(index);
			base.RemoveItem(index);
		}

		protected override void SetItem(int index, ResourceItem item)
		{
			this.ClearNotifications(index);
			this.InitializeNotifications(item);
			base.SetItem(index, item);
		}

		protected override void ClearItems()
		{
			for (int i = 0; i < this.Count; i++)
			{
				this.ClearNotifications(i);
			}
			base.ClearItems();
		}

		private void OnItemPropertyChanged(object sender, PropertyChangedEventArgs args)
		{
			if (args.PropertyName == "IsChecked")
			{
				ResourceItem item = sender as ResourceItem;
				if (item.IsChecked)
				{
					AddToCheckedItems(item);
				}
				else
				{
					this.RemoveFromCheckedItems(item);
				}
			}
		}

		private void AddToCheckedItems(ResourceItem item)
		{
			if (!this.CheckedItems.Contains(item))
			{
				this.CheckedItems.Add(item);
				this.OnPropertyChanged(new PropertyChangedEventArgs("CheckedItems"));
			}
		}

		private void RemoveFromCheckedItems(ResourceItem item)
		{
			if (this.CheckedItems.Contains(item))
			{
				this.CheckedItems.Remove(item);
				this.OnPropertyChanged(new PropertyChangedEventArgs("CheckedItems"));
			}
		}

		private void InitializeNotifications(ResourceItem item)
		{
			item.PropertyChanged += OnItemPropertyChanged;
			if (item.IsChecked)
				this.AddToCheckedItems(item);
			item.Owner = this;
		}

		private void ClearNotifications(int index)
		{
			if (index < this.Count)
			{
				ResourceItem item = this[index];
				item.PropertyChanged -= OnItemPropertyChanged;
				this.RemoveFromCheckedItems(item);
				item.Owner = null;
			}
		}
	}
}