﻿using System.Linq;
using System.Collections.Specialized;
using System;

namespace CheckBoxesInCombo
{
	public class ResourceTypeModel : NotifyPropertyChanged
	{
		private Appointment Appointment { get; set; }
		public ResourceType ResourceType { get; set; }

		public ResourceItem SelectedItem
		{
			get
			{
				var selectedItems = this.ResourceItems.CheckedItems;
				return selectedItems.Count == 0 ? null : selectedItems[0];
			}
			set
			{
				var selectedItems = this.ResourceItems.CheckedItems;
				if (selectedItems.Count == 0 || selectedItems[0] != value)
				{
					for (int i = selectedItems.Count; i > 0; i--)
					{
						selectedItems.RemoveAt(0);
					}
					selectedItems.Add(value);
					this.OnPropertyChanged("SelectedItem");
				}
			}
		}

		public ResourceTypeModel(ResourceType resourceType, Appointment appointment)
		{
			this.Appointment = appointment;
			this.ResourceType = resourceType;

			this.ResourceItems = new ResourceItemCollection();
			this.ResourceItems.CheckedItems.CollectionChanged += CheckedItems_CollectionChanged;

			foreach (Resource resource in resourceType.Resources)
			{
				this.ResourceItems.Add(new ResourceItem() { Resource = resource, IsChecked = this.IsChecked(resource) });
			}
		}

		private void CheckedItems_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
		{
			switch (e.Action)
			{
				case NotifyCollectionChangedAction.Add:
					if (e.NewItems != null)
					{
						foreach (ResourceItem item in e.NewItems)
						{
							Resource resource = this.Appointment.Resources.FirstOrDefault(r => r.Equals(item.Resource));
							if (resource == null)
							{
								this.Appointment.Resources.Add(item.Resource);
							}
						}
					}
					break;
				case NotifyCollectionChangedAction.Remove:
					if (e.OldItems != null)
					{
						foreach (ResourceItem item in e.OldItems)
						{
							Resource resource = this.Appointment.Resources.FirstOrDefault(r => r.Equals(item.Resource));
							if (resource != null)
							{
								this.Appointment.Resources.Remove(resource);
							}
						}
					}
					break;
				case NotifyCollectionChangedAction.Replace:
					break;
				case NotifyCollectionChangedAction.Reset:
				default:
					break;
			}

		}

		private bool IsChecked(Resource resource)
		{
			return this.Appointment.Resources.Any(r => r.Equals(resource));
		}

		public ResourceItemCollection ResourceItems { get; private set; }
	}
}